package PowerCMS::Util;

use strict;
use Exporter;
@PowerCMS::Util::ISA = qw( Exporter );
use vars qw( @EXPORT_OK );
@EXPORT_OK = qw( build_tmpl save_asset upload association_link create_entry make_entry
                 write2file read_from_file move_file copy_item remove_item
                 relative2path path2relative path2url relative2url url2path
                 site_path site_url static_or_support support_dir archive_path
                 current_ts next_date prev_date valid_ts month2int send_mail get_mail
                 mime_type valid_email get_content get_feed extract_content regex_extract
                 str_replace regex_replace ftp_new ftp_get ftp_put ftp_mkdir ftp_quit
                 make_zip_archive current_user get_user current_blog uniq_filename set_upload_filename
                 get_childlen_files get_childlen_filenames file_extension file_label
                 is_ua_iPhone is_ua_mobile is_ua_keitai is_user_can is_power_edit is_cms
                 is_application is_windows is_blog is_plugin is_writable is_image
                 if_ua_iPhone if_ua_mobile if_ua_keitai if_user_can if_power_edit if_cms
                 if_application if_windows if_blog if_plugin if_writable if_image
                 uniq_array ceil floor round format_LF valid_ip get_mobile_id get_utf utf8_on
                 utf8_off to_utf8 normalize upload_callback chomp_dir add_slash
                 powercms_files_dir register_template register_templates_to
                 load_registered_template load_registered_template_for force_background_task
                 powercms_files_dir_path make_dir
               );

use MT::Util qw( epoch2ts ts2epoch offset_time_list format_ts encode_url
                 perl_sha1_digest_hex is_valid_email );
use MT::Log;
use MT::FileMgr;
use File::Basename;
use File::Spec;
use Image::Size;
use File::Temp qw( tempdir ); 
use Encode;
use Encode qw( encode decode );

sub build_tmpl {
    my ( $app, $tmpl, $args, $params ) = @_;
#     my %args = ( blog => $blog,
#                  entry => $entry,
#                  category => $category,
#                  author => $author,
#                  start => 'YYYYMMDDhhmmss',
#                  end => 'YYYYMMDDhhmmss',
#                 );
#     my %params = ( foo => 'bar', # => <mt:var name="foo">
#                    bar =>'buz', # => <mt:var name="bar">
#                  );
#    my $tmpl = MT::Template->load( { foo => 'bar' } ); # or text
#    return build_tmpl( $app, $tmpl, \%args, \%params );
    if ( ( ref $tmpl ) eq 'MT::Template' ) {
        $tmpl = $tmpl->text;
    }
    $tmpl = $app->translate_templatized( $tmpl );
    require MT::Template;
    require MT::Builder;
    require MT::Template::Context;
    my $ctx = MT::Template::Context->new;
    my $blog = $args->{ blog };
    my $entry = $args->{ entry };
    my $category = $args->{ category };
    if ( (! $blog ) && ( $entry ) ) {
        $blog = $entry->blog;
    }
    if ( (! $blog ) && ( $category ) ) {
        $blog = MT::Blog->load( $category->blog_id );
    }
    my $author = $args->{ author };
    my $start = $args->{ start };
    my $end = $args->{ end };
    $ctx->stash( 'blog', $blog );
    $ctx->stash( 'blog_id', $blog->id ) if $blog;
    $ctx->stash( 'entry', $entry );
    $ctx->stash( 'page', $entry );
    $ctx->stash( 'category', $category );
    $ctx->stash( 'category_id', $category->id ) if $category;
    $ctx->stash( 'author', $author );
    if ( $start && $end ) {
        if ( ( valid_ts( $start ) ) && ( valid_ts( $end ) ) ) {
            $ctx->{ current_timestamp } = $start;
            $ctx->{ current_timestamp_end } = $end;
        }
    }
    for my $stash ( keys %$args ) {
        if (! $ctx->stash( $stash ) ) {
            if ( ( $stash ne 'start' ) && ( $stash ne 'end' ) ) { 
                $ctx->stash( $stash, $args->{ $stash } );
            }
        }
    }
    for my $key ( keys %$params ) {
        $ctx->{ __stash }->{ vars }->{ $key } = $params->{ $key };
    }
    my $build = MT::Builder->new;
    my $tokens = $build->compile( $ctx, $tmpl )
        or return $app->error( $app->translate(
            "Parse error: [_1]", $build->errstr ) );
    defined( my $html = $build->build( $ctx, $tokens ) )
        or return $app->error( $app->translate(
            "Build error: [_1]", $build->errstr ) );
    unless ( MT->version_number < 5 ) {
        $html = utf8_on( $html );
    }
    return $html;
}

sub save_asset {
    my ( $app, $blog, $params, $cb ) = @_;
#    my %params = ( file => $file,
#                   author => $author,
#                   label => $label,
#                   description => $description,
#                   parent => $parent_asset->id,
#                   object => $obj,
#                   tags => \@tags,
#                   );
#    my $asset = save_asset( $app, $blog, \%params );
    my $blog_id = $blog->id;
    my $file_path = $params->{ file };
    my $fmgr = $blog->file_mgr;
    unless ( $fmgr->exists( $file_path ) ) {
        return undef;
    }
    my $file = $file_path;
    my $author = $params->{ author };
    $author = $app->user unless ( defined $author );
    my $label = $params->{ label };
    my $description = $params->{ description };
    my $parant = $params->{ parant };
    my $obj = $params->{ object };
    my $tags = $params->{ tags };
    my $basename = File::Basename::basename( $file_path );
    my $file_ext = file_extension( $file_path );
    my $mime_type = mime_type( $file_path );
    my $class = 'file'; my $is_image;
    require MT::Asset;
    my $asset_pkg = MT::Asset->handler_for_file( $basename );
    my $asset;
    if ( $asset_pkg eq 'MT::Asset::Image' ) {
        $asset_pkg->isa( $asset_pkg );
        $class = 'image';
        $is_image = 1;
    }
    if ( $asset_pkg eq 'MT::Asset::Audio' ) {
        $asset_pkg->isa( $asset_pkg );
        $class = 'audio';
    }
    if ( $asset_pkg eq 'MT::Asset::Video' ) {
        $asset_pkg->isa( $asset_pkg );
        $class = 'video';
    }
    my $url = $file_path;
    $url =~ s!\\!/!g if if_windows();
    $url = path2url( $url, $blog );
    $url = path2relative( $url, $blog );
    $file_path = path2relative( $file_path, $blog );
    $asset = $asset_pkg->load( { blog_id => $blog_id,
                                 file_path => $file_path } );
    my $original;
    unless ( $asset ) {
        $asset = $asset_pkg->new();
    } else {
        $original = $asset->clone();
    }
    $original = $asset->clone();
    $asset->blog_id( $blog_id );
    $asset->url( $url );
    $asset->file_path( $file_path );
    $asset->file_name( $basename );
    $asset->mime_type( $mime_type );
    $asset->file_ext( $file_ext );
    $asset->class( $class );
    $asset->created_by( $author->id );
    $asset->modified_by( $author->id );
    if ( $parant ) {
        $asset->parent( $parant );
    }
    my ( $w, $h, $id );
    if ( $class eq 'image' ) {
        ( $w, $h, $id ) = imgsize( $file );
        $asset->image_width( $w );
        $asset->image_height( $h );
    }
    unless ( $label ) {
        $label = file_label( $basename );
    }
    $asset->label( $label );
    if ( $description ) {
        $asset->description( $description );
    }
    if ( $cb ) {
        $app->run_callbacks( 'cms_pre_save.asset', $app, $asset, $original )
          || return $app->errtrans( "Saving [_1] failed: [_2]", 'asset',
            $app->errstr );
    }
    $asset->set_tags( @$tags );
    $asset->save or die $asset->errstr;
    if ( $cb ) {
        $app->run_callbacks( 'cms_post_save.asset', $app, $asset, $original );
    } else {
        $app->log(
            {
                message => $app->translate(
                    "File '[_1]' uploaded by '[_2]'", $asset->file_name,
                    $author->name,
                ),
                level    => MT::Log::INFO(),
                class    => 'asset',
                blog_id  => $blog_id,
                category => 'new',
            }
        );
    }
    # my @fstats = stat( $file );
    # my $bytes = $fstats[7];
    if ( $obj ) {
        if ( $obj->id ) {
            require MT::ObjectAsset;
            my $object_asset = MT::ObjectAsset->get_by_key( { 
                                                              blog_id => $obj->blog_id,
                                                              asset_id => $asset->id,
                                                              object_id => $obj->id,
                                                              object_ds => $obj->datasource,
                                                            } );
            unless ( $object_asset->id ) {
                $object_asset->save or die $object_asset->errstr;
            }
        }
    }
    my $res = upload_callback( $app, $blog, $asset, $id ) if $cb;
    return $asset;
}

sub upload {
    my ( $app, $blog, $name, $dir, $params ) = @_;
#    my %params = ( object => $obj,
#                   author => $author,
#                   rename => 1,
#                   label => 'foo',
#                   description => 'bar',
#                   format_LF => 1,
#                   singler => 1,
#                   no_asset => 1,
#                   );
#    my $upload = upload( $app, $blog, $name, $dir, \%params );
    my $obj = $params->{ object };
    my $rename = $params->{ 'rename' };
    my $label = $params->{ label };
    my $format_LF = $params->{ format_LF };
    my $singler = $params->{ singler };
    my $no_asset = $params->{ no_asset };
    my $description = $params->{ description };
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    my $q = $app->param;
    my @files = $q->upload( $name );
    my @assets;
    for my $file ( @files ) {
        my $orig_filename = $file;
        my $file_label = file_label( $orig_filename );
        $orig_filename = set_upload_filename( $orig_filename );
        my $out = File::Spec->catfile( $dir, $orig_filename );
        if ( $rename ) {
            $out = uniq_filename( $out );
        }
        $dir =~ s!/$!! unless $dir eq '/';
        unless ( $fmgr->exists( $dir ) ) {
            $fmgr->mkpath( $dir ) or return MT->trans_error( "Error making path '[_1]': [_2]",
                                    $out, $fmgr->errstr );
        }
        my $temp = "$out.new";
        local *OUT;
        open ( OUT, ">$out" ) or die "Can't open $out!";
        binmode ( OUT );
        while( read ( $file, my $buffer, 1024 ) ) {
            $buffer = format_LF( $buffer ) if $format_LF;
            print OUT $buffer;
        }
        close ( OUT );
        $fmgr->rename( $temp, $out );
        my $user = $params->{ author };
        $user = current_user( $app ) unless defined $user;
        if ( $no_asset ) {
            if ( $singler ) {
                return $out;
                last;
            }
            push ( @assets, $out );
        } else {
            if ( ( $user ) && ( $blog ) ) {
                my %params = ( file => $out,
                               author => $user,
                               label => ( $label || $file_label ),
                               description => $description,
                               object => $obj,
                               );
                my $asset = save_asset( $app, $blog, \%params, 1 ) or die;
                if ( $singler ) {
                    return $asset;
                    last;
                }
                push ( @assets, $asset ) if defined $asset;
            }
        }
    }
    return \@assets;
}

sub association_link {
    my ( $app, $author, $role, $blog ) = @_;
    require MT::Association;
    my $assoc = MT::Association->link( $author => $role => $blog );
    if ( $assoc ) {
        my $log = MT::Log->new;
        my $msg = { message => $app->translate(
                    '[_1] registered to the blog \'[_2]\'',
                    $author->name,
                    $blog->name
                ),
                level    => MT::Log::INFO(),
                class    => 'author',
                category => 'new',
                blog_id => $blog->id,
        };
        if ( ref $app =~ /^MT::App::/ ) {
            $msg->{ ip } = $app->remote_ip;
            if ( my $user = $app->user ) {
                $log->author_id( $user->id );
            }
        }
        $log->set_values( $msg );
        $log->save or die $log->errstr;
        return $assoc;
    }
    return undef;
}

sub create_entry {
    my ( $app, $blog, $args, $params, $cb ) = @_;
#     my @categories = MT::Category->load( { foo => 'bar' } );
#     my @tags = ( 'foo', 'bar' );
#     %args = ( title => 'foo',
#               text  => 'bar',
#               author_id => 1, # or current user
#               status => MT::Entry::RELEASE(), # or blog's status_default
#               customfield_basename => 'buz',
#               tags => \@tags, # string array
#               category_id => 1, # or categories => \@categories,
#              );
#     %params = ( rebuildme => 1,
#                 dependencies => 1,
#                 no_save => 1,
#                 background => 1,
#              );
#     my $entry = create_entry( $app, $blog, \%args, \%params[, $cb] );
    require MT::Entry;
    my $entry; my $is_new; my $original;
    if ( $args->{ id } ) {
        $entry = MT::Entry->load( $args->{ id } );
    }
    unless ( defined $entry ) {
        $entry = MT::Entry->new;
        $is_new = 1;
    } else {
        $original = $entry->clone_all();
    }
    $entry->blog_id ( $blog->id );
    my $clumns = $entry->column_names;
    my $professional; my $fields;
    eval { require CustomFields::Field;
           require CustomFields::BackupRestore;
           };
    unless ( $@ ){
        $professional = 1;
    }
    for my $key ( keys %$args ) {
        if ( grep( /^$key$/, @$clumns ) ) {
            $entry->$key( $args->{ $key } );
        } else {
            if ( $key eq 'tags' ) {
                my $tags = $args->{ $key };
                $entry->set_tags( @$tags );
            } elsif ( $key eq 'category_id' ) {
            } elsif ( $key eq 'categories' ) {
            } else {
                if ( $professional ) {
                    my $cf = CustomFields::Field->load( { blog_id => [ 0, $blog->id ],
                                                          basename => $key } );
                    if ( defined $cf ) {
                        $fields->{ $key } = $args->{ $key };
                    }
                }
            }
        }
    }
    my $user = current_user( $app );
    unless ( $entry->author_id ) {
        if ( defined $user ) {
            $entry->author_id( $user->id );
        } else {
            return undef;
        }
    } else {
        unless ( $user ) {
            require MT::Author;
            $user = MT::Author->load( $entry->author_id );
            unless ( defined $user ) {
                return undef;
            }
        }
    }
    unless ( $entry->created_by ) {
        $entry->created_by( $entry->author_id );
    }
    unless ( $entry->modified_by ) {
        $entry->modified_by( $entry->author_id );
    }
    unless ( $entry->status ) {
        $entry->status( $blog->status_default );
    }
    unless ( $entry->allow_comments ) {
        $entry->allow_comments( $blog->allow_comments_default );
    }
    unless ( $entry->allow_pings ) {
        $entry->allow_pings( $blog->allow_pings_default );
    }
    unless ( $entry->class ) {
        $entry->class( 'entry' );
    }
    unless ( $entry->authored_on ) {
        $entry->authored_on( current_ts( $blog ) );
    }
    if ( $entry->allow_pings ) {
        unless ( $entry->atom_id ) {
            $entry->atom_id( $entry->make_atom_id() );
        }
    }
    if ( $cb ) {
        $app->run_callbacks( 'cms_pre_save.' . $entry->class, $app, $entry, $original );
    }
    if ( $params->{ no_save } ) {
        return $entry;
    }
    $entry->save or die $entry->errstr;
    my @saved_cats;
    if ( $args->{ category_id } ) {
        require MT::Placement;
        my $place = MT::Placement->get_by_key( { blog_id => $blog->id,
                                                 category_id => $args->{ category_id },
                                                 entry_id => $entry->id,
                                                 is_primary => 1,
                                             } );
        $place->save or die $place->errstr;
        push ( @saved_cats, $args->{ category_id } );
    }
    if ( $args->{ categories } ) {
        my $categories = $args->{ categories };
        my $i = 1;
        for my $category ( @$categories ) {
            my $is_primary = 0;
            if ( ( $i == 1 ) && (! $args->{ category_id } ) ) {
                $is_primary = 1;
            }
            my $place = MT::Placement->get_by_key( { blog_id => $blog->id,
                                                     category_id => $category->id,
                                                     entry_id => $entry->id,
                                                     is_primary => $is_primary,
                                                 } );
            $place->save or die $place->errstr;
            push ( @saved_cats, $category->id );
            $i++;
        }
    }
    if (! $is_new ) {
        my @placement = MT::Placement->load( blog_id => $blog->id,
                                             entry_id => $entry->id, );
        for my $place ( @placement ) {
            my $place_id = $place->id;
            if ( (! scalar @saved_cats ) || (! grep( /^$place_id$/, @saved_cats ) ) ) {
                $place->remove or die $place->errstr;
            }
        }
    }
    if ( $professional ) {
        CustomFields::BackupRestore::_update_meta( $entry, $fields );
    }
    $entry->clear_cache();
    if ( $cb ) {
        $app->run_callbacks( 'cms_post_save.' . $entry->class, $app, $entry, $original );
    }
    if ( $params->{ rebuildme } ) {
        my $dependencies = $params->{ dependencies };
        if ( $entry->status == MT::Entry::RELEASE() ) {
            if ( $params->{ background } ) {
                force_background_task(
                    sub { $app->rebuild_entry( Entry => $entry->id,
                                               BuildDependencies => $dependencies );
                    }
                );
            } else {
                MT::Util::start_background_task(
                    sub { $app->rebuild_entry( Entry => $entry->id,
                                               BuildDependencies => $dependencies );
                    }
                );
            }
        }
    }
    return $entry;
}

sub make_entry {
    return create_entry( @_ );
}

sub write2file {
    my ( $path, $data, $type, $blog ) = @_;
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    if ( $blog ) {
        $path = relative2path( $path, $blog );
    }
    my $dir = dirname( $path );
    $dir =~ s!/$!! unless $dir eq '/';
    unless ( $fmgr->exists( $dir ) ) {
        $fmgr->mkpath( $dir ) or return MT->trans_error( "Error making path '[_1]': [_2]",
                                $path, $fmgr->errstr );
    }
    $fmgr->put_data( $data, "$path.new", $type );
    $fmgr->rename( "$path.new", $path );
    if ( $fmgr->exists( $path ) ) {
        return 1;
    } else {
        return 0;
    }
}

sub read_from_file {
    my ( $path, $type, $blog ) = @_;
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    if ( $blog ) {
        $path = relative2path( $path, $blog );
    }
    unless ( $fmgr->exists( $path ) ) {
       return '';
    }
    my $data = $fmgr->get_data( $path, $type );
    return $data;
}

sub move_file {
    my ( $from, $to, $blog ) = @_;
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    if ( $blog ) {
        $from = relative2path( $from, $blog );
        $to = relative2path( $to, $blog );
    }
    my $dir = dirname( $to );
    $dir =~ s!/$!! unless $dir eq '/';
    unless ( $fmgr->exists( $dir ) ) {
        $fmgr->mkpath( $dir ) or return MT->trans_error( "Error making path '[_1]': [_2]",
                                $to, $fmgr->errstr );
    }
    $fmgr->rename( $from, $to );
}

sub copy_item {
    my ( $from, $to, $blog ) = @_;
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    if ( $blog ) {
        $from = relative2path( $from, $blog );
        $to = relative2path( $to, $blog );
    }
    my $dir = dirname( $to );
    $dir =~ s!/$!! unless $dir eq '/';
    unless ( $fmgr->exists( $dir ) ) {
        $fmgr->mkpath( $dir ) or return MT->trans_error( "Error making path '[_1]': [_2]",
                                $to, $fmgr->errstr );
    }
    if ( File::Copy::Recursive::rcopy ( $from, $to ) ) {
        return 1;
    } else {
        return 0;
    }
}

sub remove_item {
    my ( $remove, $blog ) = @_;
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    if ( $blog ) {
        $remove = relative2path( $remove, $blog );
    }
    unless ( $fmgr->exists( $remove ) ) {
        return 0;
    }
    if ( -f $remove ) {
        return $fmgr->delete( $remove );
    } elsif ( -d $remove ) {
        File::Path::rmtree( [ $remove ] );
        unless ( -d $remove ) {
            return 1;
        }
    }
    return 0;
}

sub relative2path {
    my ( $path, $blog ) = @_;
    my $app = MT->instance();
    my $static_file_path = static_or_support();
    my $archive_path = archive_path( $blog );
    my $site_path = site_path( $blog );
    $path =~ s/%s/$static_file_path/;
    $path =~ s/%r/$site_path/;
    if ( $archive_path ) {
        $path =~ s/%a/$archive_path/;
    }
    return $path;
}

sub path2relative {
    my ( $path, $blog ) = @_;
    my $app = MT->instance();
    my $static_file_path = quotemeta( static_or_support() );
    my $archive_path = quotemeta( archive_path( $blog ) );
    my $site_path = quotemeta( site_path( $blog ) );
    $path =~ s/$static_file_path/%s/;
    $path =~ s/$site_path/%r/;
    if ( $archive_path ) {
        $path =~ s/$archive_path/%a/;
    }
    if ( $path =~ m!^https{0,1}://! ) {
        my $site_url = quotemeta( site_url( $blog ) );
        $path =~ s/$site_url/%r/;
    }
    return $path;
}

sub path2url {
    my ( $path, $blog ) = @_;
    # $path = relative2path( $path, $blog );
    my $site_path = quotemeta ( site_path( $blog ) );
    my $site_url = site_url( $blog );
    $path =~ s/^$site_path/$site_url/;
    return $path;
}

sub relative2url {
    my ( $path, $blog ) = @_;
    return path2url( relative2path( $path,$blog ), $blog );
}

sub url2path {
    my ( $url, $blog ) = @_;
    my $site_url = quotemeta ( site_url( $blog ) );
    my $site_path = site_path( $blog );
    $url =~ s/^$site_url/$site_path/;
    return $url;
}

sub site_path {
    my $blog = shift;
    my $site_path = $blog->archive_path;
    $site_path = $blog->site_path unless $site_path;
    return chomp_dir( $site_path );
}

sub archive_path {
    my $blog = shift;
    my $archive_path = $blog->archive_path;
    return chomp_dir( $archive_path );
}

sub site_url {
    my $blog = shift;
    my $site_url = $blog->site_url;
    if ( $site_url =~ /(.*)\/$/ ) {
        $site_url = $1;
    }
    return $site_url;
}

sub static_or_support {
    my $app = MT->instance();
    my $static_or_support;
    if ( MT->version_number < 5 ) {
        $static_or_support = $app->static_file_path;
    } else {
        $static_or_support = $app->support_directory_path;
    }
    return $static_or_support;
}

sub support_dir {
    my $app = MT->instance();
    my $support_dir;
    if ( MT->version_number < 5 ) {
        $support_dir = File::Spec->catdir( $app->static_file_path, 'support' );
    } else {
        $support_dir = $app->support_directory_path;
    }
    return $support_dir;
}

sub current_ts {
    my $blog = shift;
    my @tl = offset_time_list( time, $blog );
    my $ts = sprintf "%04d%02d%02d%02d%02d%02d", $tl[5]+1900, $tl[4]+1, @tl[3,2,1,0];
    return $ts;
}

sub next_date {
    my ( $blog, $ts ) = @_;
    $ts = ts2epoch( $blog, $ts );
    $ts += 86400;
    return epoch2ts( $blog, $ts );
}

sub prev_date {
    my ( $blog, $ts ) = @_;
    $ts = ts2epoch( $blog, $ts );
    $ts -= 86400;
    return epoch2ts( $blog, $ts );
}

sub valid_ts {
    my $ts = shift;
    return 0 unless ( $ts =~ m/^[0-9]{14}$/ );
    my $year = substr( $ts, 0, 4 );
    my $month = substr( $ts, 4, 2 );
    my $day = substr( $ts, 6, 2 );
    my ( @mlast ) = ( 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 );
    if ( $month < 1 || 12 < $month ) {
        return 0;
    }
    if ( $month == 2 ) {
        if ( ( ( $year % 4 == 0 ) && ( $year % 100 != 0 ) ) || ( $year % 400 == 0 ) ) {
            $mlast[1]++;
        }
    }
    if ( $day < 1 || $mlast[$month-1] < $day ) {
        return 0;
    }
    my $hour = substr( $ts, 8, 2 );
    my $min = substr( $ts, 10, 2 );
    my $sec = substr( $ts, 12, 2 );
    if ( ( $hour < 25 ) && ( $min < 61 ) && ( $sec < 61 ) ) {
        return 1;
    } else {
        return 0;
    }
}

sub month2int {
    my $month = shift;
    if ( $month =~ m/^Jan/i ) { return 1 };
    if ( $month =~ m/^Feb/i ) { return 2 };
    if ( $month =~ m/^Mar/i ) { return 3 };
    if ( $month =~ m/^Apr/i ) { return 4 };
    if ( $month =~ m/^Mar/i ) { return 5 };
    if ( $month =~ m/^Jun/i ) { return 6 };
    if ( $month =~ m/^Jul/i ) { return 7 };
    if ( $month =~ m/^Aug/i ) { return 8 };
    if ( $month =~ m/^Sep/i ) { return 9 };
    if ( $month =~ m/^Oct/i ) { return 10 };
    if ( $month =~ m/^Nov/i ) { return 11 };
    if ( $month =~ m/^Dec/i ) { return 12 };
    return 0;
}

sub send_mail {
    my ( $from, $to, $subject, $body,
                     $cc, $bcc, $params4cb ) = @_; #old interface
    my ( $args, $params );
    if ( ref $from eq 'HASH' ) { # new interface
        $args    = $from;
        $params  = $to;
        $from    = $args->{ from };
        $to      = $args->{ to };
        $subject = $args->{ subject };
        $body    = $args->{ body };
        $cc      = $args->{ cc };
        $bcc     = $args->{ bcc };
    }
    else {
        $args = {
            from    => $from,
            to      => $to,
            subject => $subject,
            body    => $body,
            cc      => $cc,
            bcc     => $bcc,
        };
        $params = $params4cb;
    }
    return unless defined( $subject );
    return unless defined( $body );
    return unless ( $from && $to && $subject ne '' && $body ne '' );
    $params = { key => 'default' } unless defined $params;
    $params->{ key } = 'default' unless defined $params->{ key };
    my $app = MT->instance();
    my $mgr = MT->config;
    my $enc = $mgr->PublishCharset;
    my $mail_enc = uc ( $mgr->MailEncoding || $enc );
    $body = MT::I18N::encode_text( $body, $enc, lc $mail_enc );
    return unless
        $app->run_callbacks( ( ref $app ) . '::pre_send_mail', $app, \$args, \$params );
    my %head;
    %head = (
        To => $to,
        From => $from,
        Subject => $subject,
        ( ref $cc eq 'ARRAY' ? ( Cc => $cc ) : () ),
        ( ref $bcc eq 'ARRAY' ? ( Bcc => $bcc ) : () ),
    );
    require MT::Mail;
    if ( is_application( $app ) ) {
        force_background_task(
           sub { MT::Mail->send( \%head, $body )
                or return ( 0, "The error occurred.", MT::Mail->errstr ); } );
    } else {
        MT::Mail->send( \%head, $body )
                or return ( 0, "The error occurred.", MT::Mail->errstr );
    }
}

sub get_mail {
    my ( $server, $id, $passwd, $protocol, $delete ) = @_;
    my $app = MT->instance();
    eval { require Net::POP3;
           require MIME::Parser; };
    unless ( $@ ){
        my $pop3 = Net::POP3->new( $server ) 
            or die "Could not open pop3. ( Server : $server )";
        next unless ( defined $pop3 );
        my $login = ( lc( $protocol ) eq 'apop' ) ? 'apop' : 'login';
        $login = $pop3->$login( $id, $passwd );
        return undef unless $login;
        my $messages = $pop3->list();
        my $charset = $app->config( 'PublishCharset' );
        $charset = 'utf8' if $charset eq 'UTF-8';
        $charset = 'euc-jp' if $charset eq 'EUC-JP';
        $charset = 'shift-jis' if $charset eq 'Shift_JIS';
        my $tempdir = $app->config( 'TempDir' );
        my @emails;
        for my $id ( sort ( keys %$messages ) ) {
            mkdir( $tempdir, 0777 ) unless ( -d $tempdir );
            my $message = $pop3->get( $id );
            my $parser = new MIME::Parser;
            my $workdir = tempdir ( DIR => $tempdir );
            $parser->output_dir( $workdir );
            my $entity = $parser->parse_data( $message );
            my $header = $entity->head;
            my $from = $header->get( 'From' );
            my $to = $header->get( 'to' );
            my $subject = $header->get( 'Subject' );
            $subject = encode( $charset, decode( 'MIME-Header', $subject ) );
            unless ( MT->version_number < 5 ) {
                $subject = utf8_on( $subject );
            }
            $from = encode( $charset , decode( 'MIME-Header', $from ) );
            $from =~ s/\n//sg;
            $subject =~ s/\n//sg if $subject;
            $from = $1 if ( $from =~ /<(.{1,})>/ );
            my $body = ''; my @f;
            if ( $entity->is_multipart ) {
                opendir( DIR, $workdir );
                my @files = readdir( DIR );
                closedir( DIR );
                for my $file ( @files ) {
                    if ( $file !~ /^\./ ) {
                        if ( $file =~ /^msg\-[0-9]{1,}\-[0-9]{1,}\.txt/ ) {
                            $body .= read_from_file( File::Spec->catfile ( $workdir, $file ) );
                        } else {
                            push ( @f, File::Spec->catfile ( $workdir, $file ) );
                        }
                    }
                }
            } else {
                $body = $entity->bodyhandle;
                $body = $body->as_string;
            }
            $body = encode( $charset, decode( 'iso-2022-jp', $body ) );
            unless ( MT->version_number < 5 ) {
                $body = utf8_on( $body );
            }
            my $mail = { from => $from,
                         subject => $subject,
                         body    => $body,
                         files   => \@f,
                         directory => $workdir };
            push ( @emails, $mail );
            $pop3->delete( $id ) if $delete;
        }
        $pop3->quit;
        return \@emails;
    } else {
        return undef;
    }
    return undef;
}

sub make_zip_archive {
    my ( $directory, $out, $files ) = @_;
    eval { require Archive::Zip };
    if ( $@ ){
        return undef;
    }
    $directory =~ s!/$!!;
    unless ( $files ) {
        @$files = get_childlen_filenames( $directory );
    }
    $directory = quotemeta( $directory );
    my $archiver = Archive::Zip->new();
    for my $file ( @$files ) {
        my $new = $file;
        $new =~ s/^$directory//;
        $archiver->addFile( $file, $new );
    }
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    my $dir = dirname( $out );
    $dir =~ s!/$!! unless $dir eq '/';
    unless ( $fmgr->exists( $dir ) ) {
        $fmgr->mkpath( $dir ) or return undef;
    }
    return $archiver->writeToFileNamed( $out );
}

sub current_user {
    my $app = shift;
    my $user;
    eval { $user = $app->user };
    if ( $@ ) {
        return undef;
    } else {
        return $user if defined $user;
    }
    return undef;
}

sub get_user {
    my $app = shift;
    my $user; my $sess;
    if ( is_application( $app ) ) {
        eval { $user = $app->user };
        unless ( defined $user ) {
            eval { ( $sess, $user ) = $app->get_commenter_session() };
            unless ( defined $user ) { 
                if ( $app->param( 'sessid' ) ) {
                    my $sess = MT::Session->load ( { id => $app->param( 'sessid' ),
                                                     kind  => 'US' } );
                    if ( defined $sess ) {
                       my $sess_timeout = $app->config->UserSessionTimeout;
                       if ( ( time - $sess->start ) < $sess_timeout ) {
                            $user = MT::Author->load( { name => $sess->name } );
                            $sess->start( time );
                            $sess->save or die $sess->errstr;
                        }
                    }
                }
            }
        }
    }
    return $user if defined $user;
    return undef;
}

sub current_blog {
    my $app = shift;
    my $blog;
    eval { $blog = $app->blog };
    if ( $@ ) {
        return undef;
    } else {
        return $blog;
    }
}

sub get_content {
    my ( $uri, $utf8, $file ) = @_;
    my $app = MT->instance();
    eval { require LWP::UserAgent };
    if ( $@ ){
        return undef;
    }
    my $remote_ip;
    eval { $remote_ip = $app->remote_ip; };
    my $agent;
    if ( $remote_ip ) {
        $agent = "Mozilla/5.0 (Power CMS for MT X_FORWARDED_FOR:$remote_ip)";
    } else {
        $agent = "Mozilla/5.0 (Power CMS for MT)";
    }
    my $protcol = $1 if $uri =~ /(^.*?):/;
    my $ua = LWP::UserAgent->new;
    $ua->agent( $agent );
    my $proxy = MT->instance->config->PingProxy;
    $ua->proxy( $protcol, $proxy ) if ( $protcol && $proxy );
    my $no_proxy = MT->instance->config->PingNoProxy;
    $ua->no_proxy( split( /,\s*/, $no_proxy ) ) if $no_proxy;
    my $timeout = MT->instance->config->PingTimeout;
    $ua->timeout( $timeout ) if $timeout;
    my $req = HTTP::Request->new( GET => $uri );
    my $res = $ua->request( $req );
    if ( $res ) {
        my $content = $res->content;
        if ( $utf8 ) {
            $content = # MT::I18N::encode_text( $content, undef, 'utf-8' );
            $content = to_utf8( $content );
        }
        if ( $file ) {
            write2file( $file, $content ) or return undef;
        }
        unless ( MT->version_number < 5 ) {
            $content = utf8_on( $content ) if $utf8;
        }
        return $content;
    } else {
        return undef;
    }
    # my $code = $res->code;
    # my @response = [ $content, $code ];
    # return \@response;
    return undef;
}

sub get_feed {
    my ( $uri, $utf8 ) = @_;
    return undef unless $uri;
    require MT::Feeds::Lite;
    my $lite  = MT::Feeds::Lite->fetch( $uri ) or return ''; 
    my $title = $lite->find_title( $lite->feed );
    my $link  = $lite->find_link ( $lite->feed );
    my $entries = $lite->entries;
    my $count = scalar @$entries;
    if ( $utf8 ) {
        $title = to_utf8( $title );
        $link  = to_utf8( $link );
        unless ( MT->version_number < 5 ) {
            $title = utf8_on( $title );
            $link  = utf8_on( $link );
        }
    }
    my %feed = (
        LITE    => $lite,
        TITLE   => $title,
        LINK    => $link,
        ENTRIES => \%$entries,
        COUNT   => $count,
    );
    return \%feed;
}

sub extract_content {
    my ( $start, $end, $data, $cont ) = @_;
    $start = quotemeta( $start );
    $end   = quotemeta( $end );
    if ( $data =~ /($start)(.*?)($end)/s ) {
        if (! $cont ) {
            return $2;
        } else {
            return $1 . $2 . $3;
        }
    }
    return '';
}

sub regex_extract {
    my ( $pattern, $data ) = @_;
    if ( $pattern =~ m!^/! ) {
        if ( $pattern !~ /[\(\)]/ ) {
            $pattern =~ s!^/!/(!;
            $pattern =~ s!/([^/]*$)!)/$1!;
        }
        $pattern =~ s!^/!/.*?!;
        $pattern =~ s!/([^/]*$)!\.*\$/$1!;
    }
    return regex_replace( $pattern, '$1', $data );
}

sub str_replace {
    my ( $from, $to, $data ) = @_;
    $from = quotemeta( $from );
    $data =~ s/$from/$to/g;
    return $data;
}

sub regex_replace {
    my ( $pattern, $replace, $data ) = @_;
    require MT::Template::Context;
    my $ctx = MT::Template::Context->new;
    my $val = [ $pattern, $replace ];
    if ( MT->version_number < 5 ) {
        $data = MT::Template::Context::_fltr_regex_replace( $data, $val, $ctx );
    } else {
        require MT::Template::Tags::Filters;
        $data = MT::Template::Tags::Filters::_fltr_regex_replace( $data, $val, $ctx );
    }
    return $data;
}

sub ftp_new {
    my ( $server, $account, $password, $param ) = @_;
    eval { require Net::FTP };
    if ( $@ ){
        return undef;
    }
    my $ftp = Net::FTP->new( $server, %$param ) or die;
    my $login = $ftp->login( $account, $password ) or return undef;
    return $ftp;
}

sub ftp_get {
    my ( $ftp, $cwd, $rfile, $lfile, $mode ) = @_;
    $mode = 'binary' unless $mode;
    $ftp->$mode;
    $ftp->cwd( $cwd ) or return undef;
    return $ftp->get( $rfile, $lfile );
}

sub ftp_put {
    my ( $ftp, $cwd, $file, $mode ) = @_;
    $mode = 'binary' unless $mode;
    $ftp->$mode;
    $ftp->cwd( $cwd ) or return undef;
    return $ftp->put( $file );
}

sub ftp_mkdir {
    my ( $ftp, $cwd, $dir, $mode ) = @_;
    $ftp->cwd( $cwd ) or return undef;
    return $ftp->mkdir( $dir );
}

sub ftp_quit {
    my $ftp = shift;
    return $ftp->quit;
}

sub uniq_filename {
    my $file = shift;
    my $dir = dirname( $file );
    my $tilda = quotemeta( '%7E' );
    $file =~ s/$tilda//g;
    $file = File::Spec->catfile( $dir, set_upload_filename( $file ) );
    return $file unless ( -f $file );
    my $file_extension = file_extension( $file );
    my $base = $file;
    $base =~ s/(.{1,})\.$file_extension$/$1/;
    $base = $1 if ( $base =~ /(^.*)_[0-9]{1,}$/ );
    my $i = 0;
    do { $i++;
         $file = $base . '_' . $i . '.' . $file_extension;
       } while ( -e $file );
    return $file;
}

sub set_upload_filename {
    my $file = shift;
    $file = File::Basename::basename( $file );
    my $ctext = encode_url( $file );
    if ( $ctext ne $file ) {
        unless ( MT->version_number < 5 ) {
            $file = utf8_off( $file );
        }
        my $extension = file_extension( $file );
        my $ext_len = length( $extension ) + 1;
        if ( eval { require Digest::MD5; 1; } ) {
            $file = Digest::MD5::md5_hex( $file );
        } else {
            $file = perl_sha1_digest_hex( $file );
        }
        $file = substr ( $file, 0, 255 - $ext_len );
        $file .= '.' . $extension;
    }
    return $file;
}

sub get_childlen_files {
    return get_childlen_filenames( @_ );
}

sub get_childlen_filenames {
    my ( $directory, $pattern ) = @_;
    my @wantedFiles;
    require File::Find;
    if ( $pattern ) {
        if ( $pattern =~ m!^(/)(.+)\1([A-Za-z]+)?$! ) {
            $pattern = $2;
            if ( my $opt = $3 ) {
                $opt =~ s/[ge]+//g;
                $pattern = "(?$opt)" . $pattern;
            }
            my $regex = eval { qr/$pattern/ };
            if ( defined $regex ) {
                my $command = 'File::Find::find ( sub { push ( @wantedFiles, $File::Find::name ) if ( /' . $pattern. '/ ) && -f ; }, $directory );';
                eval $command;
                if ( $@ ) {
                    return undef;
                }
            } else {
                return undef;
            }
        }
    } else {
        File::Find::find ( sub { push ( @wantedFiles, $File::Find::name ) unless (/^\./) || ! -f ; }, $directory );
    }
    return @wantedFiles;
}

sub uniq_array {
    my $array = shift;
    my %hash  = ();
    for my $value ( @$array ){
        $hash{ $value } = 1;
    }
    return (
        keys %hash
    );
}

sub ceil {
   my $var = shift;
   my $a = 0;
   $a = 1 if ( $var > 0 and $var != int ( $var ) );
   return int ( $var + $a );
}

sub floor {
    my $var = shift;
    return int( $var );
}

sub round {
    my $var = shift;
    return int( $var + 0.5 );
}

sub format_LF {
    my $data = shift;
    $data=~ s/\r\n/\n/gs;
    $data =~ s/\r/\n/g;
    return $data;
}

sub if_ua_keitai {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    if ( ( $ua =~ /iPhone/ ) || ( $ua =~ /iPod/ ) ) {
        return 0;
    }
    my @browswer = split( m!/!, $ua );
    $ua = $browswer[0];
    if ( $ua eq 'DoCoMo' ) {
        return 1;
    }
    if ( ( $ua eq 'SoftBank' ) || ( $ua eq 'Vodafone' ) ) {
        return 1;
    }
    if ( $ua =~ /UP\.Browser/ ) {
        return 1;
    }
    return 0;
}

sub if_ua_mobile {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    if ( if_ua_iPhone ( $app ) ) {
        return 1;
    }
    return if_ua_keitai( $app );
}

sub if_ua_iPhone {
    my $app = shift;
    my $ua = $app->get_header( 'User-Agent' );
    if ( ( $ua =~ /iPhone/ ) || ( $ua =~ /iPod/ ) ) {
        return 1;
    }
    return 0;
}

sub if_user_can {
    my ( $blog, $user, $permission ) = @_;
#     my $app = MT->instance();
#     my $permissions = $app->registry( 'permissions' );
    $permission = 'can_' . $permission;
    my $perm = $user->is_superuser;
    unless ( $perm ) {
        if ( $blog ) {
            $perm = $user->permissions( $blog->id )->$permission;
        }
    }
    return $perm;
}

sub if_power_edit {
    my $app = shift;
    if ( if_cms( $app ) ) {
        my $return_args = $app->param( 'return_args' );
        if ( $return_args =~ /&is_power_edit=1/ ) {
            return 1;
        }
    }
    return 0;
}

sub if_application {
    my $app = shift;
    if ( ( ref $app ) =~ /^MT::App::/ ) {
        return 1;
    } else {
        return 0;
    }
}

sub if_cms {
    my $app = shift;
    if ( ref $app eq 'MT::App::CMS' ) {
        return 1;
    } else {
        return 0;
    }
}

sub if_windows {
    if ( $^O eq 'MSWin32' ) {
        return 1;
    }
    return 0;
}

sub if_blog {
    my $app = shift;
    if ( current_blog( $app ) ) {
        return 1;
    }
    return 0;
}

sub if_plugin {
    my $component = shift;
    if ( $component ) {
        my $plugin = MT->component( $component );
        if ( defined $plugin ) {
            return 1;
        }
    }
    return 0;
}

sub if_writable {
    my ( $path, $blog ) = @_;
    my $app = MT->instance();
    $path = File::Spec->canonpath( $path );
    my $tempdir = quotemeta( $app->config( 'TempDir' ) );
    my $importdir = quotemeta( $app->config( 'ImportPath' ) );
    my $powercms_files_dir = quotemeta( powercms_files_dir() );
    my $support_dir = quotemeta( support_dir() );
    if ( ( $path =~ /^$tempdir/ )
        || ( $path =~ /^$importdir/ )
        || ( $path =~ /^$powercms_files_dir/ )
        || ( $path =~ /^$support_dir/ ) ) {
        return 1;
    }
    if ( defined $blog ) {
        my $site_path = quotemeta( site_path( $blog ) );
        if ( $path =~ /^$site_path/ ) {
            return 1;
        }
    }
    return 0;
}

sub is_ua_iPhone {
    return if_ua_iPhone( @_ );
}

sub is_ua_mobile {
    return if_ua_mobile( @_ );
}

sub is_ua_keitai {
    return if_ua_keitai( @_ );
}

sub is_user_can {
    return if_user_can( @_ );
}

sub is_power_edit {
    return if_power_edit( @_ );
}

sub is_cms {
    return if_cms( @_ );
}

sub is_application {
    return if_application( @_ );
}

sub is_windows {
    return if_windows();
}

sub is_blog {
    return if_blog( @_ );
}

sub is_plugin {
    return if_plugin( @_ );
}

sub is_writable {
    return if_writable( @_ );
}

sub file_extension {
    my $file = shift;
    my $extension;
    if ( $file =~ /\.([^\.]*)$/ ) {
        $extension = $1;
        return lc ( $extension );
    } else {
        return '';
    }
}

sub file_label {
    my $file = shift;
    $file = File::Basename::basename( $file );
    my $file_extension = file_extension( $file );
    my $base = $file;
    $base =~ s/(.{1,})\.$file_extension$/$1/;
    return $base;
}

sub mime_type {
    my $file = shift;
    my %mime_type = (
        'css'   =>  'text/css',
        'html'  =>  'text/html',
        'htm'   =>  'text/html',
        'txt'   =>  'text/plain',
        'rtx'   =>  'text/richtext',
        'tsv'   =>  'text/tab-separated-values',
        'hdml'  =>  'text/x-hdml;charset=Shift_JIS',    
        'xml'   =>  'application/rss+xml',
        'rdf'   =>  'application/rss+xml',
        'xsl'   =>  'text/xsl',
        'mpeg'  =>  'video/mpeg',
        'mpg'   =>  'video/mpeg',
        'mpe'   =>  'video/mpeg',
        'qt'    =>  'video/quicktime',
        'avi'   =>  'video/x-msvideo',
        'movie' =>  'video/x-sgi-movie',
        'qt'    =>  'video/quicktime',
        'ice'   =>  'x-conference/x-cooltalk',
        'svr'   =>  'x-world/x-svr',
        'vrml'  =>  'x-world/x-vrml',
        'wrl'   =>  'x-world/x-vrml',
        'vrt'   =>  'x-world/x-vrt',    
        'spl'   =>  'application/futuresplash',
        'hqx'   =>  'application/mac-binhex40',
        'doc'   =>  'application/msword',
        'pdf'   =>  'application/pdf',
        'ai'    =>  'application/postscript',
        'eps'   =>  'application/postscript',
        'ps'    =>  'application/postscript',
        'ppt'   =>  'application/powerpoint',
        'rtf'   =>  'application/rtf',
        'dcr'   =>  'application/x-director',
        'dir'   =>  'application/x-director',
        'dxr'   =>  'application/x-director',
        'js'    =>  'application/x-javascript',
        'dvi'   =>  'application/x-dvi',
        'gtar'  =>  'application/x-gtar',
        'gzip'  =>  'application/x-gzip',
        'latex' =>  'application/x-latex',
        'lzh'   =>  'application/x-lha',
        'swf'   =>  'application/x-shockwave-flash',
        'sit'   =>  'application/x-stuffit',
        'tar'   =>  'application/x-tar',
        'tcl'   =>  'application/x-tcl',
        'tex'   =>  'application/x-texinfo',
        'texinfo'=> 'application/x-texinfo',
        'texi'  =>  'application/x-texi',
        'src'   =>  'application/x-wais-source',
        'zip'   =>  'application/zip',
        'au'    =>  'audio/basic',
        'snd'   =>  'audio/basic',
        'midi'  =>  'audio/midi',
        'mid'   =>  'audio/midi',
        'kar'   =>  'audio/midi',
        'mpga'  =>  'audio/mpeg',
        'mp2'   =>  'audio/mpeg',
        'mp3'   =>  'audio/mpeg',
        'ra'    =>  'audio/x-pn-realaudio',
        'ram'   =>  'audio/x-pn-realaudio',
        'rm'    =>  'audio/x-pn-realaudio',
        'rpm'   =>  'x-pn-realaudio-plugin',
        'wav'   =>  'audio/x-wav',
        'bmp'   =>  'image/bmp',
        'gif'   =>  'image/gif',
        'jpeg'  =>  'image/jpeg',
        'jpg'   =>  'image/jpeg',
        'jpe'   =>  'image/jpeg',
        'png'   =>  'image/png',
        'tiff'  =>  'image/tiff',
        'tif'   =>  'image/tiff',
        'pnm'   =>  'image/x-portable-anymap',
        'ras'   =>  'image/x-cmu-raster',
        'pnm'   =>  'image/x-portable-anymap',
        'pbm'   =>  'image/x-portable-bitmap',
        'pgm'   =>  'image/x-portable-graymap',
        'ppm'   =>  'image/x-portable-pixmap',
        'rgb'   =>  'image/x-rgb',
        'xbm'   =>  'image/x-xbitmap',
        'xpm'   =>  'image/x-pixmap',
        'xwd'   =>  'image/x-xwindowdump',
    );
    my $extension = file_extension( $file );
    my $type = $mime_type{ $extension };
    $type = 'application/octet-stream' unless $type;
    return $type;
}

sub valid_email {
    my $email = shift;
    return 0 unless is_valid_email ( $email );
    if ( $email =~ /^[^@]+@[^.]+\..+/ ) {
        return 1;
    } else {
        return 0;
    }
}

sub get_mobile_id {
    my ( $app, $to_hash ) = @_;
    my $mobile_id;
    my $user_agent = $app->get_header( 'User-Agent' );
    my @browswer = split( m!/!, $user_agent );
    my $ua = $browswer[0];
    if ( $ua eq 'DoCoMo' ) {
        if ( $user_agent =~ /^.*(ser[0-9]{11,}).*$/ ) {
            $mobile_id = $1;
        }
    } elsif ( $ua =~ /UP\.Browser/ ) {
        my $x_up_subno = $user_agent = $app->get_header( 'X_UP_SUBNO' );
        # AU
        if ( $x_up_subno ) {
            $mobile_id = $x_up_subno;
        }
    } elsif ( ( $ua eq 'SoftBank' ) || ( $ua eq 'Vodafone' ) ) {
        # Softbank
        my $x_jphone_uid = $app->get_header( 'X_JPHONE_UID' );
        if ( $x_jphone_uid ) {
            $mobile_id = $x_jphone_uid;
        }
    }
    if ( $mobile_id && $to_hash ) {
        $mobile_id = perl_sha1_digest_hex( $mobile_id );
    }
    return $mobile_id if $mobile_id;
    return '';
}

sub valid_ip {
    my ( $remote_ip, $table ) = @_;
    # valid_ip( $app->remote_ip, \@ip_table )
    return 1 if ( grep( /^$remote_ip$/, @$table ) );
    my $ip_table = join ( "\n", @$table );
    if ( $remote_ip =~ /(^[0-9]{1,}\.[0-9]{1,}\.[0-9]{1,}\.)([0-9]{1,}$)/ ) {
        my @bits = qw/ 0 126 62 30 14 6 2 /;
        my $check = quotemeta( $1 );
        my $last = $2;
        if ( $ip_table =~ /$check([0-9]{1,})\/([0-9]{1,})/ ) {
            my $begin = $1;
            my $bit = $2;
            if ( ( $begin eq '0' ) && ( $bit eq '24' ) ) {
                if ( ( 0 < $last ) && ( 255 > $last ) ) {
                    return 1;
                }
            } else {
                $bit = $bit - 24;
                my $range = $bits[ $bit ];
                my $end = $begin + $range;
                if ( ( $last >= $begin ) && ( $last <= $end ) ) {
                    return 1;
                }
            }
        }
    }
    return 0;
}

sub get_utf {
    my $text = shift;
    eval { require Unicode::Japanese; };
    if ( $@ ) {
        return undef;
    } else {
        my $t = Unicode::Japanese->new( $text, 'utf8' );
        $text = $t->getu();
    }
    return $text;
}

sub utf8_on {
    my $text = shift;
    if (! Encode::is_utf8( $text ) ) {
        Encode::_utf8_on( $text );
    }
    return $text;
}

sub utf8_off {
    my $text = shift;
    return MT::I18N::utf8_off( $text );
}

sub to_utf8 {
    my $text = shift;
    return MT::I18N::encode_text( $text, undef, 'utf-8' );
}

sub normalize {
    my $text = shift;
    require Unicode::Normalize;
    $text = Unicode::Normalize::NFKC( $text );
    return $text;
}

sub upload_callback {
    my ( $app, $blog, $asset, $id ) = @_;
    my $file = $asset->file_path;
    my @fstats = stat( $file );
    my $bytes = $fstats[7];
    my $url = $asset->url;
    $app->run_callbacks(
        'cms_upload_file.' . $asset->class,
        File  => $file,
        file  => $file,
        Url   => $url,
        url   => $url,
        Size  => $bytes,
        size  => $bytes,
        Asset => $asset,
        asset => $asset,
        Type  => $asset->class,
        type  => $asset->class,
        Blog  => $blog,
        blog  => $blog
    );
    if ( $asset->class eq 'image' ) {
        unless ( $id ) {
            my ( $w, $h );
            ( $w, $h, $id ) = imgsize( $file );
        }
        $app->run_callbacks(
            'cms_upload_image',
            File       => $file,
            file       => $file,
            Url        => $url,
            url        => $url,
            Size       => $bytes,
            size       => $bytes,
            Asset      => $asset,
            asset      => $asset,
            Height     => $asset->image_height,
            height     => $asset->image_height,
            Width      => $asset->image_width,
            width      => $asset->image_width,
            Type       => 'image',
            type       => 'image',
            ImageType  => $id,
            image_type => $id,
            Blog       => $blog,
            blog       => $blog
        );
    }
    return 1;
}

sub is_image {
    my $file = shift;
    my $basename = File::Basename::basename( $file );
    require MT::Asset;
    my $asset_pkg = MT::Asset->handler_for_file( $basename );
    if ( $asset_pkg eq 'MT::Asset::Image' ) {
        return 1;
    }
    return 0;
}

sub if_image {
    my $file = shift;
    return is_image( $file );
}

sub chomp_dir {
    my $dir = shift;
    my @path = File::Spec->splitdir( $dir );
    $dir = File::Spec->catdir( @path );
    return $dir;
}

sub add_slash {
    my $path = shift;
    return $path if $path eq '/';
    if ( $path =~ m!^https?://! ) {
        if ( $path !~ m!/$! ) {
            $path .= '/';
        }
        return $path;
    }
    $path = chomp_dir( $path );
    if ( is_windows() ) {
        $path .= '\\';
    } else {
        $path .= '/';
    }
    return $path;
}

sub powercms_files_dir {
    my $powercms_files = powercms_files_dir_path();
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    if (-d $powercms_files ) {
        if (-w $powercms_files ) {
            my $do = _create_powercms_subdir( $powercms_files );
            return chomp_dir( $powercms_files );
        } else {
            chmod ( 0777, $powercms_files );
            my $do = _create_powercms_subdir( $powercms_files );
            return chomp_dir( $powercms_files ) if (-w $powercms_files );
        }
    }
    $powercms_files =~ s!/$!! unless $powercms_files eq '/';
    unless ( $fmgr->exists( $powercms_files ) ) {
        $fmgr->mkpath( $powercms_files );
        if (-d $powercms_files ) {
            unless (-w $powercms_files ) {
                chmod ( 0777, $powercms_files );
            }
        }
    }
    unless (-d $powercms_files ) {
        return undef;
    }
    if (-w $powercms_files ) {
        my $do = _create_powercms_subdir( $powercms_files );
        return chomp_dir( $powercms_files );
    }
    return undef;
}

sub powercms_files_dir_path {
    return MT->instance()->config( 'PowerCMSFilesDir' ) ||
        File::Spec->catdir( MT->instance()->mt_dir, 'powercms_files' );
}

sub _create_powercms_subdir {
    my $powercms_files = shift;
    if (-d $powercms_files ) {
        if (-w $powercms_files ) {
            my @dirs = qw( cache backup log mail lock report );
            for my $dir ( @dirs ) {
                my $directory = File::Spec->catdir( $powercms_files, $dir );
                unless (-e $directory ) {
                    if ( make_dir( $directory ) ) {
                        unless (-w $directory ) {
                            chmod ( 0777, $directory );
                        }
                    }
                } else {
                    unless (-w $directory ) {
                        chmod ( 0777, $directory );
                    }
                }
                return 0 unless (-w $directory );
            }
        }
    }
    return 1;
}

sub make_dir {
    my $path = shift;
    return 1 if (-d $path );
    my $fmgr = MT::FileMgr->new( 'Local' ) or die MT::FileMgr->errstr;
    $path =~ s!/$!! unless $path eq '/';
    unless ( $fmgr->exists( $path ) ) {
        $fmgr->mkpath( $path );
        if (-d $path ) {
            # chmod ( 0777, $path );
            return 1;
        }
    }
}

sub _slurp {
    my ( $path ) = @_;
    require IO::File;
    my $fh = IO::File->new( $path, 'r' );
    local $/ unless wantarray;
    return <$fh>;
}

sub _abs_template_path {
    my ( $plugin, $path ) = @_;
    my $tmpl_path = File::Spec->canonpath( $path );
    unless ( File::Spec->file_name_is_absolute( $tmpl_path ) ) {
        if ( $plugin->can( 'path' ) ) {
            $tmpl_path = File::Spec->catdir( $plugin->path, 'tmpl', $tmpl_path );
        }
    }
    return $tmpl_path;
}

sub _check_template_name {
    my ( $app, $blog_id, $name ) = @_;
    unless ( MT::Template->exist( { name => $name, blog_id => $blog_id } ) ) {
        return $name; # OK ,template object with same name doesn't exist
    }
    # see MT::CMS::Template::clone_templates
    my $new_basename = $app->translate( "Copy of [_1]", $name );
    my $new_name = $new_basename;
    my $i = 0;
    while ( MT::Template->exist( { name => $new_name, blog_id => $blog_id } ) ) {
        $new_name = $new_basename . ' (' . ++$i . ')';
    }
    return $new_name;
}

sub register_template {
    my ( $identifier, $path, $params ) = @_;
    # identifier is required
    return unless ( defined( $identifier ) && $identifier ne '' ); 

    $params = ref $params eq 'HASH' ? { %$params } : {}; # for safe
    my $terms = { identifier => $identifier };
    $terms->{ blog_id } = $params->{ blog_id } ? delete $params->{ blog_id } : 0 ;
    require MT::Template;
    my $tmpl = MT::Template->get_by_key( $terms );
    return if ( $tmpl->id ); # Do nothing if already exists

    # if $path is scalar, $path is for 'text' column
    # if $path is hash must have 'text' key
    my $path_info = ref $path ? $path : { text => $path };
    return unless ( ref $path_info eq 'HASH' && defined( $path_info->{ text } ) );

    # make values for MT::Template object
    my $plugin = $params->{ component }       ? delete $params->{ component } : MT->app;
    my $name   = defined( $params->{ name } ) ? delete $params->{ name }      : $identifier;
    $name = _check_template_name( $plugin, $terms->{ blog_id }, $name );
    my %values;
    $values{ type }       = $params->{ type }       ? delete $params->{ type }       : 'custom';
    $values{ rebuild_me } = $params->{ rebuild_me } ? delete $params->{ rebuild_me } : 0;
    $values{ name }       = $plugin->translate( $name );
    for my $col ( keys( %$params ) ) {
        if ( MT::Template->has_column( $col ) ) {
            $values{ $col } = $params->{ $col };
        }
    }

    for my $col ( keys( %$path_info ) ) {
        my $tmpl_path = _abs_template_path( $plugin, $path_info->{ $col } );
        next unless -f $tmpl_path; # cannot find template file
        $values{ $col } = $plugin->translate_templatized( scalar( _slurp( $tmpl_path ) ) )
            unless exists $values{ $col }; # dont override exists column like type
    }
    return unless exists $values{ text }; # at least needs 'text' column

    $tmpl->set_values( \%values );
    $tmpl->save
        or die $tmpl->errstr;
}

sub register_templates_to {
    my ( $blog_id, $component, $templates ) = @_;
    return unless ( ref $templates eq 'HASH' );
    $blog_id ||= 0;
    return unless ( $blog_id =~ m/^\d+$/ );
    my $ret = 1;
    for my $ident ( keys( %$templates ) ) {
        my $v    = $templates->{ $ident };
        next unless ref $v eq 'HASH';
        $v = { %$v };
        my $path = delete $v->{ path };
        next unless $path;
        my %param = (
            blog_id => $blog_id,
            ( $component ? ( component => $component ) : () ),
            %$v,
        );
        $ret = 0 unless register_template( $ident, $path, \%param );
    }
    return $ret;
}

sub load_registered_template {
    my ( $identifier, $path, $params ) = @_;
    # identifier is required
    return unless ( defined( $identifier ) && $identifier ne '' ); 

    my $terms = { identifier => $identifier };
    $terms->{ blog_id } = $params->{ blog_id } ? $params->{ blog_id } : 0 ;

    my $path_info = ref $path ? $path : { text => $path };
    return unless ( ref $path_info eq 'HASH' && defined( $path_info->{ text } ) );

    require MT::Template;
    my $tmpl = MT::Template->load( $terms );
    if ( $tmpl ) {
        return $tmpl unless wantarray;
        my @ret;
        push @ret, $tmpl for keys %$path_info;
        return @ret;
    }

    # default
    my $plugin = $params->{ component } ? $params->{ component } : MT->app;
    if ( wantarray ) {
        my @ret;
        for my $col ( sort( keys( %$path_info ) ) ) {
            push( @ret, _abs_template_path( $plugin, $path_info->{ $col } ) );
        }
        return @ret;
    }
    else {
        return _abs_template_path( $plugin, $path_info->{ text } );
    }
}

sub load_registered_template_for {
    my ( $blog_id, $component, $ident, $templates ) = @_;
    return unless ( ref $templates eq 'HASH' );
    $blog_id ||= 0;
    return unless ( $blog_id =~ m/^\d+$/ );
    my $v = $templates->{ $ident };
    return unless ref $v eq 'HASH';
    $v = { %$v };
    my $path = delete $v->{ path };
    return unless $path;
    my %param = (
        blog_id => $blog_id,
        ( $component ? ( component => $component ) : () ),
        %$v,
    );
    return load_registered_template( $ident, $path, \%param );
}

sub force_background_task {
    my $app = MT->instance();
    my $fource = $app->config->FourceBackgroundTasks;
    if ( $fource ) {
        my $default = $app->config->LaunchBackgroundTasks;
        $app->config( 'LaunchBackgroundTasks', 1 );
        my $res = MT::Util::start_background_task( @_ );
        $app->config( 'LaunchBackgroundTasks', $default );
        return $res;
    } else {
        return MT::Util::start_background_task( @_ );
    }
}

1;

__END__

=head1 NAME

PowerCMS::Util - Powrer CMS for MT utility functions

=head1 SYNOPSIS

    use lib qw( plugins/PowerCMS/lib );
    use PowerCMS::Util qw( function1 function2 );

=head2 build_tmpl( $app, $tmpl, \%args [, \%params ] )

コンテクストをセットしてテンプレートのビルド( 再構築 )結果を返します。

=head3 args

=over 4

=item * $app  - ( required )

=item * $tmpl - ( required )

MT::Templateオブジェクト又は文字列のいずれかを指定します。

=item * \%args - ( optional )

MTオブジェクトの各オブジェクト ( MT::Blog,MT::Category, MT::Entry, MT::Author ) および
タイムスタンプ ( start,end ) を指定します。例:

%args = ( blog     => $blog,     # optional
          entry    => $entry,    # optional
          category => $category, # optional
          author   => $author,   # optional
          start    => 'YYYYMMDDhhmmss', # optional
          end      => 'YYYYMMDDhhmmss',  # optional
        );

=item * \%params - ( optional )

指定した値は<mt:var name="key">で取り出すことができます。例:

    my %params = ( foo => 'bar', # => <mt:var name="foo">
                   bar => 'buz', # => <mt:var name="bar">
                 );

=back

=head2 save_asset( $app, $blog, $params, $cb )

アイテム ( MT:Assetオブジェクト ) を作成して保存します。

=head3 args

=over 4

=item * $app  - ( required )

=item * $blog - ( required )

=item * \%params - ( required )

%params = ( file => $file,     # required
            author => $author, # required ( MT::Authorオブジェクト )
            label => $label,   # optional
            description => $description,  # optional
            parent => $parent_asset->id,  # optional
            object => $obj,    # optional ( 指定したオブジェクトとアイテムを関連付けます )
            tags => \@tags,    # optional ( 指定するタグの配列 ( 文字列 ) )
          );

=item * $cb - ( optional )

アイテムの直前に cms_pre_save.asset, 保存直後に cms_post_save.asset 及び
cms_upload_file( cms_upload_image )コールバックをコールします。

=back

=head2 upload( $app, $blog, $name, $dir, \%params )

ブラウザからアップロードされたファイルを保存してアイテムに登録します。
戻り値はアイテム( MT::Assetオブジェクト )の配列です。
$params{ no_asset } = 1を指定した場合はファイルパスの配列を返します。
$params{ singler }  = 1を指定した場合配列ではなくスカラー型で値を返します。

=head3 args

=over 4

=item * $app  - ( required )

=item * $blog - ( required )

=item * $name - ( required )

input要素のname属性を指定します。(<input type="file" name="[name]" />)

=item * $dir  - ( required )

ファイルの保存場所をフルパスで指定します。

=item * \%params - ( required )

%params = ( object => $obj,    # optional ( 指定したオブジェクトと保存したアイテムを関連付けます )
            author => $author, # optional ( MT::Authorオブジェクト/省略時は現在のユーザー )
            rename => 1,       # optional ( ファイルが存在する場合上書きせずに別名を付けます )
            label => 'foo',    # optional ( アイテムの名前 )
            description => 'bar', # optional ( アイテムの説明 )
            format_LF => 1,    # optional ( 改行コードをLFに揃えます )
            singler   => 1,    # optional ( アップロードされるファイルが1点の場合に指定 )
            no_asset  => 1,    # optional ( アイテムを保存せずにファイルのみを保存します )
           );

=back

=head2 association_link( $app, $author, $role, $blog )

指定したブログに対してユーザーにロールを割り当てます。戻り値はMT::Associationオブジェクトです。

=head3 args

=over 4

=item * $app    - ( required )

=item * $author - ( required )

=item * $role   - ( required )

=item * $blog   - ( required )

=back

=head2 create_entry( $app, $blog, \%args, \%params[, $cb] )

MT::Entryオブジェクトを作成して保存します。
$params{ no_save } = 1を指定した場合MT::Entryオブジェクトのみを返します( 保存は行いません )。
この場合、カテゴリーや関連オブジェクトはセットされないことに注意してください。
$params{ rebuildme } = 1を指定した場合保存後に再構築を実行します。

=head3 args

=over 4

=item * $app    - ( required )

=item * $blog   - ( required )

=item * \%args  - ( required )

MT::Entryオブジェクトの各カラムの値及びカスタムフィールド、カテゴリ、タグを指定します。例:

%args = ( title => 'foo', # optional ( その他のカラム 例:text_more,excerptも同様に指定できます )
          text  => 'bar', # optional ( 一部のカラムでは指定がない場合ブログのデフォルト値が利用されます )
          author_id => 1, # optional ( 指定がない場合現在のユーザー )
          status => MT::Entry::RELEASE(), # optional ( 指定がない場合ブログのデフォルト )
          customfield_basename => 'buz',  # optional ( customfield_+フィールドのbasename )
          tags => \@tags, # optional ( 指定するタグの配列 ( 文字列 ) )
          category_id => 1, # optional ( カテゴリが一つの場合( カテゴリのID ) )
          categories => \@categories, # optional ( カテゴリが複数の場合( MT::Categoryオブジェクトの配列 ) )
         );

id( エントリーのID )を指定した場合、エントリーが存在すれば上書きします。
複数のカテゴリを指定した場合最初のカテゴリがプライマリカテゴリに指定されます。

=item * \%params - ( optional )

保存と再構築に関する指定を行います。

%params = ( rebuildme => 1,    # optional 保存後に再構築を行います
            dependencies => 1, # optional 再構築時にインデックス等の関連アーカイブを再構築します
            no_save => 1,      # optional 再構築時にインデックス等の関連アーカイブを再構築します
            background => 1,   # optional バックグラウンドプロセスで再構築を実行します。
         );

=item * $cb - ( optional )

エントリーの保存前に cms_pre_save.entry( page ) エントリーの保存後に cms_pre_save.entry( page )
コールバックをコールします。

=back

=head2 make_entry( $app, $blog, \%args, \%params[, $cb] )

create_entryへのエイリアスです。

=head2 write2file ( $path, $data, $type, $blog )

フルパス ( フルパス ) とデータを指定してファイルを生成します。

=head3 args

=over 4

=item * $path   - ( required )

=item * $data   - ( required )

=item * $type   - ( optional )

'upload' 又は 'output' ( 省略の場合'output' );

=item * $blog   - ( optional )

$blogを指定した場合相対パス ( %r や %s を含むパス ) の使用が可能です。

=back

=head2 read_from_file( $path, $type, $blog )

ファイルの内容を返します。

=head3 args

=over 4

=item * $path   - ( required )

=item * $type   - ( optional )

'upload' 又は 'ascii' ( 省略の場合'ascii' );

=item * $blog   - ( optional )

$blogを指定した場合相対パス ( %r や %s を含むパス ) の使用が可能です。

=back

=head2 move_file( $from, $to, $blog )

$from から $to へファイルを移動します。
$blogを指定した場合相対パス ( %r や %s を含むパス ) の使用が可能です。

=head2 copy_item( $from, $to, $blog )

$from から $to へファイル ( 又はフォルダ ) をコピーします。
$blogを指定した場合相対パス ( %r や %s を含むパス ) の使用が可能です。

=head2 remove_item( $remove, $blog )

$remove ( フルパス ) のファイル ( フォルダ ) を削除します。
$blogを指定した場合相対パス ( %r や %s を含むパス ) の使用が可能です。

=head2 relative2path( $path, $blog )

相対パス ( %r や %s を含むパス ) をフルパスに変換します。

=head2 path2relative( $path, $blog )

フルパスを相対パス ( %r や %s を含むパス ) に変換します。

=head2 path2url( $path, $blog )

フルパスからURLを生成します。

=head2 relative2url( $path, $blog )

相対パス ( %r や %s を含むパス ) をURLに変換します。

=head2 url2path( $url, $blog )

URLからサーバーのフルパスを生成します。

=head2 site_path( $blog )

ブログのサイト・パスを常に末尾の / ( スラッシュ ) を省略して返します。
アーカイブ・パスの指定がある場合はアーカイブ・パスを返します。

=head2 archive_path( $blog )

ブログのアーカイブ・パスを常に末尾の / ( スラッシュ ) を省略して返します。

=head2 site_url( $blog )

ブログのURLを常に末尾の / ( スラッシュ ) を省略して返します。
末尾の / を含む完全なURLを返す場合は add_slash 関数をあわせて利用してください。

=head2 static_or_support()

%s が示すディレクトリ ( MT4ではスタティックパス、MT5ではサポートディレクトリのパス ) を返します。

=head2 support_dir()

サポートディレクトリのパスを返します。

=head2 current_ts( $blog )

現在のタイムスタンプを YYYYMMDDhhmmss 形式で返します。

=head2 next_date( $blog, $ts )

$ts ( YYYYMMDDhhmmss ) の24時間後のタイムスタンプを返します。

=head2 prev_date( $blog, $ts )

$ts ( YYYYMMDDhhmmss ) の24時間前のタイムスタンプを返します。

=head2 valid_ts( $ts )

$ts ( YYYYMMDDhhmmss ) の書式が妥当で且つ存在している日時の場合 1 を返します。

=head2 month2int( $month )

月を表す文字列( 例: January, Jan, jan ) から数字を返します。

=head2 send_mail( $from, $to, $subject, $body
                  [, \@cc, \@bcc, \%params_for_callback ] )

=head2 send_mail( \%args, \%params_for_callback )

指定の $to へメールを送信します。
$from, $to, $subject, $body のいずれかが空の場合はメール送信せずにリターンします。
また、実際の送信前にpre_send_mailコールバックを\%args 形式で呼び出します。
コールバックが0を返した場合は、メール送信せずにリターンします。

=head3 args

=over 4

=item * $from - ( required )

=item * $to   - ( required )

=item * $subject - ( required )

=item * $body - ( required )

=item * $cc   - ( optional )

=item * $bcc  - ( optional )

=item * \%params_for_callbacks - ( optional )

呼び出し側でコールバックされたときに渡すものを自由に指定します。
以下のようにkeyを入れてください。keyがない場合は、'default'が自動的に入ります。例:

%params_for_callbacks = (
    key => '呼び出しを特定する文字列',
    another params ...
);

=item * \%args - ( required )

第2の呼び出しインターフェース引数

%args = (
    from    => $from,    # required
    to      => $to,      # required
    subject => $subject, # required
    body    => $body,    # required
    cc      => \@cc,     # optional
    bcc     => \@bcc,    # optional
);

=back

=head3 trigger callbacks

=over 4

=item * pre_send_mail( $app, \%args, \%spec )

%args の中の特定のキーの値を変更することで、後続の実際のメール送信対象を変更可能です。
コールバックが偽を返すとメール送信が行われません。

=back

=head2 register_template( $identifier, $path, \%param )

=head2 register_template( $identifier, \%path_info, \%param )

テンプレートモジュールを登録する

register_template( 'hoge', 'path/to/tmpl' );
register_template( 'hoge', { text => 'path/to/tmpl', subject => 'anotherpath' } );
register_template( 'hoge', 'path/to/tmpl', { blog_id => '3' } );
register_template( 'hoge', 'path/to/tmpl', { component => MT->component('Hoge') } );
register_template( 'hoge', 'path/to/tmpl', { type => 'email' } );
register_template( 'hoge', 'path/to/tmpl', { name => 'Ho Ge' } );

登録キー名,ファイルパス,その他MT::Templateの作成に必要なパラメータを渡したら
未登録ならオブジェクトを作成します。

=head3 args

=over 4

=item * identifier - ( required ) テンプレートを特定するキー

=item * path - ( required ) 登録するテンプレートファイルパス

テンプレートオブジェクトのtextカラムに入る内容の元になるファイルのパスを指定します。
パスは$params->{ component }->pathのtmpl下の相対パスか絶対パスで指定可能です。
textカラム以外の場所にも登録したい場合は、

{ text => 'textカラム用のファイルパス', subject => 'subjectカラム用のファイルパス' }

という形で、ハッシュリファレンスを渡すことにより、別のファイルを指定することが可能です。
ハッシュリファレンスを渡す場合はtextキーは必須になります。

=item * params - (optional) デフォルト以外の動作を指定する場合のパラメータ

下記以外のパラメータを指定した場合は、テンプレートの該当カラムに入れるものとして
動作します。

=over 4

=item * blog_id - ( optional; default 0 )

テンプレートを検索・登録するブログID

=item * component - ( optional; default MT->app )

翻訳,テンプレートパス決定に使用するプラグイン

=item * type - ( optional; default "custom" )

登録するタイプ

=item * name - ( optional; default identifier )

テンプレート登録名

=item * rebuild_me - ( optional; default 0 )

=back

=back

=head2 register_templates_to( $blog_id, $component, \%templates )

特定のブログにテンプレートを一式登録する。例:

my %info = {
    hoge => { path => 'path/to/tmpl' },
    hage => { path => 'path/to/tmpl', blog_id => 0 },
    hige => { path => 'path/to/tmpl', type => 'email' },
};

register_templates_to( $blog_id, MT->component( 'Hoge' ), \%info );

=head3 引数

=over 4

=item * blog_id - ( optional )

register_templateのデフォルトのblog_idとして利用します。

=item * component - ( optional )

register_templateのデフォルトのcomponentとして利用します。

=item * templates - ( optional )

各テンプレートの登録情報の入ったハッシュリファレンスです。

=over 4

=item * path - ( required )

register_templateの第2引数に使用します。

=item * その他のキー - ( optional )

register_templateの第3引数に使用します。

ただし、blog_id,componentに関しては、なければ指定のデフォルトが使用されます。

=back

=back

=head2 load_registered_template( $identifier, $path, \%param )

register_template互換の引数を受け取ります。
登録されたテンプレートオブジェクトがあればそれを、なければファイルパスを返します。
リストコンテキストでは、キーでソートされたリストを返します。
複数値の設定に対してスカラコンテキストでの取得はtextのものを返します。例:

my $tmpl = load_registered_template( 'hoge', 'path/to/tmpl' ); 
my ( $subject, $body_text ) = load_registered_template( 'hoge', { text => 'path/to/tmpl', subject => 'anotherpath' } ); 
my $body_text = load_registered_template( 'hoge', { text => 'path/to/tmpl', subject => 'anotherpath' } ); 

=head2 load_registered_template_for( $blog_id, $component, $identifier, \%templates )

register_template_to に近い引数を受け取ります。
blog_id と componentにデフォルトを設定して load_registered_template を呼び出します。

my %templates_info = (...);

my $tmpl = load_registered_template_for( $blog_id, MT->component('Hoge'), 'hoge', \%templates_info ); 

=cut
